/*
 * Desenvolvido para a disciplina Programacao 2
 * Curso de Bacharelado em Cincia da Computao
 * Departamento de Sistemas e Computao
 * Universidade Federal da Paraba
 *
 * Copyright (C) 2001 Universidade Federal da Paraba.
 * No redistribuir sem permisso.
 */
package p1.aplic.mancala.jogo;

/**
 * Este classe representa um buraco do tabuleiro do jogo de mancala.
 *
 * @author   Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.0
 * <br>
 * Copyright (C) 2001 Universidade Federal da Paraba.
 */
public class Buraco {
  /**
   * O nmero que identifica do buraco (de 0 a 13)
   */
  private int nmero;
  /**
   * O nmero de sementes no buraco
   */
  private int numSementes;

  /**
   * Construtor de um buraco com dada identificao
   *
   * @param   nmero   A identificao do buraco (de 0 a 13).
   */
  Buraco(int nmero) {
    this(nmero, 0);
  }

  /**
   * Construtor de um buraco com dada identificao e com sementes iniciais.
   *
   * @param   nmero   A identificao do buraco (de 0 a 13).
   * @param   numSementes   O nmero de sementes a colocar inicialmente no buraco.
   */
  Buraco(int nmero, int numSementes) {
    this.nmero = nmero;
    this.numSementes = numSementes;
  }

  /**
   * Informa quantas sementes esto no buraco.
   *
   * @return  O nmero de sementes no buraco.
   */
  public int getNmeroDeSementes() {
    return numSementes;
  }

  /**
   * Informa a identificao do buraco.
   *
   * @return  A identificao do buraco (de 0 a 13)
   */
  public int getNmero() {
    return nmero;
  }

  /**
   * Adiciona sementes ao buraco.
   *
   * @param   numSementes   O nmero de sementes a adicionar ao buraco.
   */
  void adicionaSementes(int numSementes) {
    this.numSementes += numSementes;
  }

  /**
   * Remove sementes do buraco.
   *
   * @param   numSementes   O nmero de sementes a remover do buraco.
   */
  void removeSementes(int numSementes) {
    this.numSementes -= numSementes;
  }

  /**
   * Representa o buraco como string
   *
   * @return  Um string representando o buraco.
   */
  public String toString() {
    return "Buraco " + nmero + ", " + getNmeroDeSementes() + " sementes";
  }
}

